--******************************************************************************************
-- 			simple user buttons, usb controller and lcd display functionality test
-- 										version: 1.00.a
--										last changes 08.04.2005
--
--	desc: A short string is send over usb to PC and to LCD if a user push button is pressed.
-- note: max. clock frequency (clk) is 5 MHz
--
--******************************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity top_level is
	port (
			usb_data			: out std_logic_vector(7 downto 0);			-- data bus
			usb_RD_n			: out std_logic;									-- read (low active)
			usb_WR			: inout std_logic;								-- write
			usb_TXE_n		: in std_logic;									-- transmit ready (low active)
			usb_SIWU			: out std_logic;									-- send immediate / wake up

			sys_clk			: in std_logic;									-- system clock 0 (60 MHz)
			button			: in std_logic_vector(3 downto 0));			-- user push buttons
end top_level;

architecture Behavioral of top_level is

	component usb_string_mem
		port (
		addr: IN std_logic_VECTOR(8 downto 0);
		clk: IN std_logic;
		dout: OUT std_logic_VECTOR(7 downto 0));
	end component;

	COMPONENT clock
	PORT(
		CLKIN_IN : IN std_logic;          
		CLKDV_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
	END COMPONENT;

	signal button_delayed : std_logic_vector(3 downto 0);
	signal usb_address : std_logic_vector(8 downto 0);
	signal new_write : std_logic;
	signal CLKIN_IBUFG_OUT : std_logic;
	signal CLK0_OUT : std_logic;
	signal LOCKED : std_logic;
	signal CLK : std_logic;														-- max 5 MHz!

begin
	usb_RD_n <= '1';
	usb_SIWU <= '1';

	usb_strings : usb_string_mem
	port map(addr => usb_address,
				clk  => CLK,
				dout => usb_data);

	Inst_clock: clock
	port map(																		-- clock generator, clk = sys_clk/16
		CLKIN_IN => SYS_CLK,
		CLKDV_OUT => CLK,
		CLKIN_IBUFG_OUT => CLKIN_IBUFG_OUT,
		CLK0_OUT => CLK0_OUT,
		LOCKED_OUT => LOCKED);

	process(CLK, LOCKED)
	begin
		if LOCKED = '0' then														-- reset until dll locked in
			usb_WR <= '0';
			usb_address <= "000000000";
			new_write <= '1';

		  	button_delayed <= "0000";
		elsif CLK'event and CLK = '1' then
			usb_WR <= '0';

			-- detect if a push button is pressed or released
			button_delayed <= button;
			if	button /= button_delayed then									-- push button pressed or released?
				case (button) is													-- select string to be written 
	     			when "0000" =>
						new_write <= '0';
	     			when "0001" =>
						usb_address <= "001100000";
						new_write <= '1';											-- start usb write process
			      when "0010" =>
						usb_address <= "001110000";
						new_write <= '1';											-- start usb write process
			      when "0100" =>
						usb_address <= "010000000";
						new_write <= '1';											-- start usb write process
			      when "1000" =>
						usb_address <= "010010000";
						new_write <= '1';											-- start usb write process
			      when others =>													-- do nothing if several buttons are pressed at the same time
						new_write <= '0';
			   end case;
			end if;

			-- USB write
			if usb_WR='0' and new_write='1' and usb_TXE_n = '0' then	-- write next byte if USB FIFO not full
				usb_WR <= '1';
			end if;

			if usb_WR='1' and new_write='1' then							-- increment adress pointer
				usb_address <= usb_address + "000000001";
			end if;

			if (usb_address = "001011111" or usb_address = "001101110" or usb_address = "001111110" or usb_address = "010001110" or usb_address = "010011110") and usb_WR='1' then	-- stop when whole string is written
				usb_address <= "000000000";
				new_write <= '0';
			end if;

		end if;
	end process;
end Behavioral;